function [G_list, G_logp, naccept, ntotal] = MH_sample_graph(S, D, I_mat, proposal_type, n_burn_in, n_thinning, G_init, verbose)
  if ~exist('verbose','var')
    verbose = true;
  end

  [m n] = size(D);
  G_list = cell(S, 1);
  G_logp = zeros(S, 1);

  G = G_init;
  [~, logp_fam] = compute_marginal_likelihood(G, zeros(0, n), sparse(0, n), D, I_mat);
  ntotal = 0;

  tt = tic;
  for i = 1:n_burn_in
    [G, logp_fam, iter] = move(G, logp_fam); 
    ntotal = ntotal + iter;
  end
  if verbose
    fprintf('MH_sample_graph: burn in complete, '); toc(tt)
  end
  
  nphase = 10;
  phase_len = floor(S / nphase);

  tt = tic;
  for s = 1:S
    G_list{s} = G;
    G_logp(s) = sum(logp_fam);
    if rem(s, phase_len) == 0
      if verbose
        fprintf('MH_sample_graph: %d/%d complete, ', s / phase_len, nphase); toc(tt);
      end
      tt = tic;
    end

    if s < S
      for i = 1:n_thinning + 1
        [G, logp_fam, iter] = move(G, logp_fam); 
        ntotal = ntotal + iter;
      end
    end
  end

  naccept = n_burn_in + (S - 1) * (n_thinning + 1);

  function [G_new, logp_fam_new, iter] = move(G, logp_fam)
    iter = 0;
    while true
      iter = iter + 1;
      if strcmp(proposal_type, 'mallows')
        [o_new, logQ_ratio] = MH_proposal_mallows(G.order, 1);
        G_new = struct('n', G.n, 'order', o_new, 'rank', order2rank(o_new));
        [~, logp_fam_new] = compute_marginal_likelihood(G_new, zeros(0, n), sparse(0, n), D, I_mat);
        logp_ratio = sum(logp_fam_new) - sum(logp_fam);
      elseif strcmp(proposal_type, 'bounded_deg')
        if ~isfield(G, 'anc')
          G.anc = adj2anc(G.adj);
        end     
        [G_new, logQ_ratio, updated_inds] = MH_proposal_bounded_deg(G, 5);
        
        old_logp = sum(logp_fam(updated_inds));
        logp_fam_new = logp_fam;
        for j = updated_inds(:)'
          [X_j, y_j] = get_family_specific_data(G_new, D, I_mat, j);
          posterior1 = compute_prior_single(size(X_j, 2) - 1);
          posterior2 = compute_posterior_for_family(X_j, y_j, posterior1);
          
          logp_fam_new(j) = -.5 * size(X_j, 1) * log(2 * pi);
          logp_fam_new(j) = logp_fam_new(j) + .5 * (logdet(posterior1.lambda) - logdet(posterior2.lambda));
          logp_fam_new(j) = logp_fam_new(j) + posterior1.alpha * log(posterior1.beta);
          logp_fam_new(j) = logp_fam_new(j) - posterior2.alpha * log(posterior2.beta);
          logp_fam_new(j) = logp_fam_new(j) + gammaln(posterior2.alpha);
          logp_fam_new(j) = logp_fam_new(j) - gammaln(posterior1.alpha);
        end

        logp_ratio = sum(logp_fam_new(updated_inds)) - old_logp;
      else
        error('Undefined proposal distribution');
      end

      if rand() < exp(min(0, logp_ratio + logQ_ratio))
        break;
      end
    end
  end
end
