function log_p = compute_marginal_likelihood_single(G, posterior1, D, I)
  posterior2 = repmat(struct('mu',[],'lambda',[],'alpha',[],'beta',[],'lambda_inv',[],'lambda_logdet',[]), G.n, 1);
  I_mat = logical(zeros(1, G.n));
  if ~isempty(I)
    I_mat([I.var]) = true;
  end

  for j = 1:G.n
    if I_mat(j)
      posterior2(j) = posterior1(j);
    else
      if isfield(G, 'order')
        parent_inds = G.order(1:G.rank(j)-1);
      else
        parent_inds = find(G.adj(:,j));
      end

      x = [1, D(parent_inds)]';
      y = D(j);

      w = posterior1(j).lambda_inv * x;
      scale = 1 + x' * w;

      posterior2(j).lambda_inv = posterior1(j).lambda_inv - w * w' / scale;
      posterior2(j).lambda_logdet = posterior1(j).lambda_logdet + log(scale);

      pri = posterior1(j);
      posterior2(j).lambda = x * x' + pri.lambda;
      posterior2(j).mu = posterior2(j).lambda_inv * (pri.lambda * pri.mu + x * y);
      posterior2(j).alpha = pri.alpha + .5;
      posterior2(j).beta = pri.beta + .5 * (y * y + pri.mu' * pri.lambda * pri.mu ...
        - posterior2(j).mu' * posterior2(j).lambda * posterior2(j).mu);
    end
  end

  num_not_clamped = sum(~I_mat);

  log_p = 0;
  log_p = log_p - (num_not_clamped / 2) * log(2 * pi);
  for j = 1:G.n
    log_p_j = 0;
    log_p_j = log_p_j + (posterior1(j).lambda_logdet - posterior2(j).lambda_logdet) / 2;
    log_p_j = log_p_j + posterior1(j).alpha * log(posterior1(j).beta);
    log_p_j = log_p_j - posterior2(j).alpha * log(posterior2(j).beta);
    log_p_j = log_p_j + gammaln(posterior2(j).alpha);
    log_p_j = log_p_j - gammaln(posterior1(j).alpha);

    log_p = log_p + log_p_j;
  end
end
