%% batch_select_method: 'active' or 'random' (type of learner)
%% ntrial: positive integer (number of trials)
%% output_file: name of MAT file to save results to
%%
%% [Example] run('active', 5, 'out_active.mat')
%%
function run(batch_select_method, ntrial, output_file)

  addpath code

  ws = load('data/10node_net.mat');
  
  %% Set up parameters
  par = struct;
  par.T = 20; % Number of iterations
  par.S = 1000; % Number of graph samples
  par.aux_param = struct;
  par.aux_param.R = 100; % Number of samples to estimate experimental outcome
  par.aux_param.MH_init_burnin = 10000; % Burn-in time for MH sampler
  par.aux_param.MH_init_thinning = 100; % Thinning interval for MH sampler
  par.aux_param.MH_update = 100; % Number of propagations for updating each graph sample
  par.batch_size = 1; % Number of experiments to perform per iteration
  par.batch_select_method = batch_select_method;
  par.proposal_type = 'bounded_deg'; % Type of proposal distribution
  
  %% Run iterative learning
  stat = cell(ntrial, 1);
  for t = 1:ntrial
    fprintf('[Trial #%d]\n', t);
    stat{t} = iterative_learning(ws.D, ws.I_mat, par.T, par.S, par.batch_size, par.batch_select_method, par.proposal_type, par.aux_param, ws.aux_data);
  end
  save(output_file, 'par', 'stat');

end
